@@ -3,6 +3,7 @@
 #' @description Extracts information from fitted CoTiMA objects to (re-)crearte list of primary studies originally created with \code{\link{ctmaPrep}}
 #'
 #' @param ctmaFitObject  ctmaFitObject
+#' @param reUseEmprawData  whether data should be transferred (will be re-used in subsequent fit attempts)
 #'
 #' @examples
 #' newStudyList <- ctmaFitToPrep(CoTiMAInitFit_3)
@@ -11,10 +12,11 @@
 #'
 #' @return list that could be used for fitting new CoTiMA models with \code{\link{ctmaInit}} or \code{\link{ctmaFit}}.
 #'
-ctmaFitToPrep <- function(ctmaFitObject=NULL)
+ctmaFitToPrep <- function(ctmaFitObject=NULL, reUseEmprawData=FALSE)
 {
   if (is.null(ctmaFitObject$originalStudyNo)) {
     if (!is.null(ctmaFitObject$studyList)) {
+      tmp <- ctmaFitObject
       ctmaFitObject <- ctmaFitObject$studyList
     }
   }
@@ -30,6 +32,9 @@ ctmaFitToPrep <- function(ctmaFitObject=NULL)
   newStudyList$n.studies  <- length(newStudyList$rawData)
   newStudyList$summary <- newStudyList
 
+  # CHD 13.6.2023
+  if (reUseEmprawData == TRUE) newStudyList$emprawList <- lapply(tmp$emprawList, function(x) x[])
+
   class(newStudyList) <-  "CoTiMAFit"
 
   invisible(newStudyList)
