@@ -1,12 +1,14 @@
-#' ctmaShapeRawDataFiles
+#' ctmaShapeRawData
 #'
 #' @description Raw data objects are re-shaped (dealing with missing time points, wrong time intervals etc)
 #'
 #' @param dataFrame  an R object containing data
+#' @param id  the identifier of subjects if data are in long format
 #' @param inputDataFrameFormat  "wide" or "long"
 #' @param inputTimeFormat  "time" (default) or "delta"
 #' @param missingValues  Missing value indicator, e.g., -999 or NA (default)
 #' @param n.manifest  Number of process variables (e.g, 2 in a bivariate model)
+#' @param manifest.per.latent n.manifest per latent factor. Frequently 1 manifest per latent, but e.g. c(2,3,1) also possible for 6 manifest loading on 3 latents
 #' @param Tpoints Number of time points in the data frame
 #' @param allInputVariablesNames  vector of all process variable names, time dependent predictor names, time independent predictor names, and names of times/deltas. Only required if the dataFrame does not have column names.
 #' @param orderInputVariablesNames  = "names" vs "time" (e.g., names: X1, X2, X3, Y1, Y2, X3 vs time: X1, Y1, X2, Y2, ... ). For ctsem/CoTiMA, the output file will order by time.
@@ -20,7 +22,9 @@
 #' @param outputTIpredNames  Will become "TI" if not specified
 #' @param outputTimeVariablesNames "time" (default)
 #' @param outputTimeFormat "time" (default) or "delta"
-#' @param scaleTime  = A scalar that is used to multiply the time variable. Typical use is rescaling primary study time to the time scale use in other primary studies. For example, scaleTime=1/(60 x 60 x 24 x 365.25) rescales time provided in seconds (frequent case when imported from SPSS) into years (60sec x 60min x 24hrs x 365.25days incl. leap years).
+#' @param scaleTime A scalar that is used to multiply the time variable. Typical use is rescaling primary study time to the time scale use in other primary studies. For example, scaleTime=1/(60 x 60 x 24 x 365.25) rescales time provided in seconds (frequent case when imported from SPSS) into years (60sec x 60min x 24hrs x 365.25days incl. leap years).
+#' @param standardization the way to standardize possible raw ("none", "withinTimeA", "withinTimeB", "withinColumn", "withinPerson", or "overall"). Only applies if the list for specifying raw data information contains the list element 'standardize=TRUE'. 'WithinTimeA' standardizes within time points and deletes cases with missing T0 data. 'WithinTimeB' does not delete cases, and in subsequent ctsem or CoTiMA applications the user is adviced to use the argument 'sameInitialTimes=TRUE'.
+#' @param keepTimePoints Do not fill any gaps in the data (no left shift).
 #' @param minInterval A parameter (default = 0.0001) supplied to ctIntervalise. Set to smaller values than any possible observed measurement interval, but larger than 0.0001. The value is used for indicating unavailable time interval information (caused by missing values) because NA is technically not possible for time intervals.
 #' @param minTolDelta Set, e.g. to 1/24, to delete variables from time points that are too close (e.g., 1hr; or even before) after another time point. Could be useful to delete values generated by unreliable responding, e.g., in diary studies. Note that minTolDelta applies to the time intervals AFTER the scaleTime argument has applied (i.e., scaleTime may need adaptation for each primary study, but minTolDelta does not).
 #' @param maxTolDelta Set, e.g., to 7, to delete variables from time points that are too far after another time point (e.g., 7 days, if all participants should have responed within a week). Note that maxTolDelta applies to the time intervals AFTER the scaleTime argument has applied (i.e., scaleTime may need adaptation for each primary study, but minTolDelta does not).
@@ -56,6 +60,7 @@
 #'
 #' @importFrom  ctsem ctWideToLong ctDeintervalise
 #' @importFrom  utils head
+#' @importFrom  stats diffinv
 #'
 #' @export ctmaShapeRawData
 #'
@@ -63,11 +68,13 @@
 #'
 ctmaShapeRawData <- function(
     dataFrame=NULL,
+    id=NULL,
     inputDataFrameFormat=NULL,
     inputTimeFormat="time",
 
     missingValues=NA,
     n.manifest=NULL,
+    manifest.per.latent=NULL,
     Tpoints=NULL,
 
     allInputVariablesNames=NULL,
@@ -93,11 +100,20 @@ ctmaShapeRawData <- function(
     min.val.n.Vars=1,
     min.val.Tpoints=1,
 
+    standardization='none',
+    keepTimePoints = FALSE,
     experimental=FALSE
 ) {
   # some checks
   {
 
+    # standardization
+    standardization <- tolower(standardization)
+    if (!( standardization %in% c("none", "withintimea", "withintimeb", "withincolumn", "withinperson", "overall"))) {
+      ErrorMsg <- "\nThe standardization argument hast to be one out of c(\"none\", \"withintimeA\", \"withintimeB\", \"withincolumn\", \"withinperson\", \"overall\"))). \nGood luck for the next try!"
+      stop(ErrorMsg)
+    }
+
     if (!(outputTimeVariablesNames %in% c("time", "dT"))) {
       ErrorMsg <- "\nThe argument \"outputTimeVariablesNames\" is currently limited to either \"time\" or \"dT\"! \nGood luck for the next try!"
       stop(ErrorMsg)
@@ -113,7 +129,7 @@ ctmaShapeRawData <- function(
       stop(ErrorMsg)
     }
 
-    if (is.null(Tpoints)) {
+    if ((is.null(Tpoints)) & (inputDataFrameFormat == 'wide')) {
       ErrorMsg <- "\nThe (maximum) number of time points has to be specified! \nGood luck for the next try!"
       stop(ErrorMsg)
     }
@@ -123,9 +139,11 @@ ctmaShapeRawData <- function(
       stop(ErrorMsg)
     }
 
-    if ( !(orderInputVariablesNames) %in% c("names", "time")) {
-      ErrorMsg <- "\nThe argument orderInputVariablesNames has to be either \"names\" or \"time\"! \nGood luck for the next try!"
-      stop(ErrorMsg)
+    if (inputDataFrameFormat == 'wide') {
+      if ( !(orderInputVariablesNames) %in% c("names", "time")) {
+        ErrorMsg <- "\nThe argument orderInputVariablesNames has to be either \"names\" or \"time\"! \nGood luck for the next try!"
+        stop(ErrorMsg)
+      }
     }
 
     if ( !(inputTimeFormat) %in% c("time", "delta")) {
@@ -134,7 +152,7 @@ ctmaShapeRawData <- function(
     }
 
     if ( !(outputTimeFormat) %in% c("time", "delta")) {
-      ErrorMsg <- "\nThe argument inputTimeFormat has to be either \"time\" or \"delta\"! \nGood luck for the next try!"
+      ErrorMsg <- "\nThe argument outputTimeFormat has to be either \"time\" or \"delta\"! \nGood luck for the next try!"
       stop(ErrorMsg)
     }
 
@@ -178,14 +196,21 @@ ctmaShapeRawData <- function(
       message(Msg)
     }
 
+    if (is.null(manifest.per.latent)) {
+      Msg <- paste0("Note: The argument manifest.per.latent was not specified (NULL). I expect that there is an eual number of manifests per latent. \n
+                    In your case I assume you have ", n.manifest, " latent variables!  \n")
+      message(Msg)
+    }
+
+
     if ( !(is.null(allInputVariablesNames)) & (!(is.null(colnames(dataFrame)))) ) {
       if ( length(allInputVariablesNames) != length(colnames(dataFrame)) ) {
         ErrorMsg <- "\nThe argument \"allInputVariablesNames\" does not equal the no. of columns of the dataFrame provided! \nGood luck for the next try!"
         stop(ErrorMsg)
       }
       Msg <- "\nThe argument \"allInputVariablesNames\" has been provided, but the dataFrame provided has colnames, too. Take care you label variables correctly! \nGood luck for the next try!"
-        message(ErrorMsg)
-      }
+      message(ErrorMsg)
+    }
 
     if (!(is.null(minTolDelta)) & !(is.null(maxTolDelta))) {
       if (minTolDelta > maxTolDelta) {
@@ -203,22 +228,25 @@ ctmaShapeRawData <- function(
     }
 
     if (minTolDelta < minInterval) {
-        ErrorMsg <- "\nThe argument minTolDelta has been set to a smaller value than mininterval (= indicator for missing)! \nGood luck for the next try!"
-        stop(ErrorMsg)
+      ErrorMsg <- "\nThe argument minTolDelta has been set to a smaller value than mininterval (= indicator for missing)! \nGood luck for the next try!"
+      stop(ErrorMsg)
     }
 
 
     tmp1 <- length(outputVariablesNames); tmp1
     if (tmp1 < n.manifest) {tmp1 <- rep(outputVariablesNames, n.manifest) } else {tmp1 <- outputVariablesNames}
     if (all(tmp1 == tmp1[1])) tmp1 <- paste0(tmp1[1], seq(1,length(tmp1),1))
-    tmp2 <- rep("_T", 4); tmp2
-    tmp3 <- paste0(tmp1, paste0(tmp2, c(0,0,1,1))); tmp3
+    #tmp2 <- rep("_T", 4); tmp2
+    tmp2 <- rep("_T", n.manifest); tmp2
+    tmp2a <- rep(0, n.manifest); tmp2a
+    tmp2b <- rep(1, n.manifest); tmp2b
+    #tmp3 <- paste0(tmp1, paste0(tmp2, c(0,0,1,1))); tmp3
+    tmp3 <- paste0(tmp1, paste0(tmp2, c(tmp2a, tmp2b))); tmp3
     tmp3 <- paste(tmp3, collapse=" "); tmp3
     Msg <- paste0("Note: Output variable names will be ", tmp3, ", etc. \n")#)
     message(Msg)
   }
 
-
   ####################################################### Shape #######################################################
   ### Step 1 (Read raw data. Store in R-Object. Replacing missing value indicators with NA)
   tmpData <- data.frame(dataFrame)
@@ -228,24 +256,56 @@ ctmaShapeRawData <- function(
   }
   #head(tmpData)
 
+
+  ### Step 2a - (re-)label variables
+  if ( !(is.null(allInputVariablesNames)) ) {
+    colnames(dataFrame) <- allInputVariablesNames
+  }
+
   ### Step 2 - (Transpose data into wide format if they are in long format)
   {
     if (inputDataFrameFormat == "long") {
-      ErrorMsg <- "\nUnfortunetaly, long format data as input is not yet implemented. Consider using the function ctLongToWide to make wide format data frame! \nGood luck for the next try!"
-      stop(ErrorMsg)
+      if (is.null(id)) {
+        ErrorMsg <- "\nYou have to specify the id (identifier) because you provided data in long format! \nGood luck for the next try!"
+        stop(ErrorMsg)
+      }
+      #ErrorMsg <- "\nUnfortunetaly, long format data as input is not yet implemented. Consider using the function ctLongToWide to make wide format data frame! \nGood luck for the next try!"
+      #stop(ErrorMsg)
+      tmpData <- tmpData[, c(id, targetTimeVariablesNames, targetInputVariablesNames, targetInputTDpredNames, targetInputTIpredNames)]
+      tmpData <- as.data.frame(ctsem::ctLongToWide(tmpData, id=id, time=targetTimeVariablesNames,
+                                                   manifestNames = targetInputVariablesNames,
+                                                   TDpredNames=targetInputTDpredNames,
+                                                   TIpredNames=targetInputTIpredNames))
+      # determine Tpoints created
+      tmp <- grep("_T", colnames(tmpData))
+      # CHD changed 13.11.2003
+      #Tpoints <- length(tmp) / (n.manifest + length(targetInputTDpredNames) + length(targetInputTIpredNames)); Tpoints
+      Tpoints <- length(tmp) / n.manifest; Tpoints
+      # make new timeVariable names
+      targetTimeVariablesNames <- paste0("T", 0:(Tpoints-1)); targetTimeVariablesNames
+      # make new inputVariable names
+      tmp <- c()
+      for (i in 1:length(targetInputVariablesNames)) tmp <- c(tmp,   paste0(targetInputVariablesNames[i], "_T", 0:(Tpoints-1)))
+      targetInputVariablesNames <- tmp
+      # define new order of names
+      orderInputVariablesNames <- 'names'
     }
-  }
 
-  ### Step 2b - (re-)label variables
-  if ( !(is.null(allInputVariablesNames)) ) {
-    colnames(dataFrame) <- allInputVariablesNames
+    # CHD removed 10.11.2023
+    #if (standardizeWithinTime == TRUE) {
+    #  for (c in targetInputVariablesNames) {
+    #    tmpData[, c] <- scale(tmpData[, c])
+    #  }
+    #}
+
   }
 
   # Step 3 (Select the desired "target variables" (at least X and Y and time) and kick out the remaining stuff.)
   #c(targetInputVariablesNames,  targetInputTDpredNames, targetTimeVariablesNames, targetInputTIpredNames)
   tmp1 <- c(targetInputVariablesNames,  targetInputTDpredNames, targetTimeVariablesNames, targetInputTIpredNames); tmp1
   tmpData <- tmpData[, tmp1]
-  #head(tmpData)
+  #head(tmpData, 30)
+  #apply(tmpData, 2, mean, na.rm=T)
 
 
   # Step 5 (Rename & re-arrange variables: X_T0, Y_T0, X_T1, Y_T1, ... time1, time2, ...)
@@ -264,7 +324,7 @@ ctmaShapeRawData <- function(
       for (i in 1:n.TDpred) {
         generalTDpredNames <- c(generalTDpredNames, paste0("TD", i))
         for (j in 0:(Tpoints-1)) {
-        outputTDpredNames <- c(outputTDpredNames, paste0("TD", i, "_T", j)); outputTDpredNames
+          outputTDpredNames <- c(outputTDpredNames, paste0("TD", i, "_T", j)); outputTDpredNames
         }
       }
     }
@@ -286,17 +346,24 @@ ctmaShapeRawData <- function(
   # time
   allOutputTimeVariablesNames <- paste0("time", seq(0, (Tpoints-1), 1)); allOutputTimeVariablesNames
 
-  # original order
+  # CHD 4.4.23 original order
   if (orderInputVariablesNames == "names") {
+    if (is.null(manifest.per.latent)) manifest.per.latent <- rep(1, n.manifest)
+    tmp1 <- stats::diffinv(manifest.per.latent*Tpoints)+1; tmp1
+    start <- tmp1[-length(tmp1)]; start
+    end <- start + manifest.per.latent-1; end
     variableOrder <- c()
-    for (i in 1:Tpoints) {
-      variableOrder <- c(variableOrder, seq(i, n.manifest*Tpoints, Tpoints)); variableOrder
+    while(end[length(end)] <= n.manifest*Tpoints) {
+      for (m in 1:length(manifest.per.latent)) {
+        variableOrder <- c(variableOrder, start[m]:end[m])
+      }
+      start <- start + manifest.per.latent
+      end <- end + manifest.per.latent
     }
     targetInputVariablesNames <- targetInputVariablesNames[variableOrder]
   }
 
   tmpData <- tmpData[, c(targetInputVariablesNames, targetInputTDpredNames, targetTimeVariablesNames, targetInputTIpredNames)]
-  #head(tmpData)
   if (inputTimeFormat == "delta") {
     dT0 <- data.frame(matrix(0, ncol=1, nrow=dim(tmpData)[1]))
     colnames(dT0) <- "dT0"
@@ -306,7 +373,10 @@ ctmaShapeRawData <- function(
   }
   colnames(tmpData) <- c(allOutputVariablesNames, outputTDpredNames, allOutputTimeVariablesNames, outputTIpredNames)
   #head(tmpData)
+  #apply(tmpData, 2, mean, na.rm=T)
+  #apply(tmpData, 2, sd, na.rm=T)
 
+  #
   #### Step 5b (make time out of delta if necessary)
   if (inputTimeFormat == "delta") {
     if (length(targetTimeVariablesNames) >= Tpoints) {
@@ -314,15 +384,15 @@ ctmaShapeRawData <- function(
       stop(ErrorMsg)
     }
     for (i in 1:(Tpoints-1)) {
-      tmp1 <- tmpData[, paste0("time", i)] == minInterval
       tmpData[, paste0("time", i)] <- tmpData[ , paste0("time", i-1)] + tmpData[ , paste0("time", i)]
-      tmpData[tmp1, paste0("time", i)] <- 0
-      }
+      if (length(tmp1) > 0) tmpData[tmp1, paste0("time", i)] <- 0
+    }
     allOutputTimeVariablesNames <- colnames(tmpData)[grep("time", colnames(tmpData))]; allOutputTimeVariablesNames
     tmp1 <- which(tmpData[, allOutputTimeVariablesNames[-1]] == 0, arr.ind = TRUE)
     tmpData[, allOutputTimeVariablesNames[-1]][tmp1] <- NA
   }
 
+
   # check
   if ( length(grep("time", colnames(tmpData)[c(allOutputVariablesNames, outputTDpredNames, outputTIpredNames)])) > 0) {
     if (minTolDelta > maxTolDelta) {
@@ -332,6 +402,8 @@ ctmaShapeRawData <- function(
   }
 
   ## at this stage, the variables should by in the order Y1_T0, Y2_T0, ..., Y1_T1, Y2_T1, ... TD1, TD2,... time1, time2,  ... TI1, TI2, ...
+  #head(tmpData)
+
 
   # Step 6: Delete variables from time points for which no time stamp is available (without time information, ctsem is impossible)
   counter <- -1
@@ -359,7 +431,6 @@ ctmaShapeRawData <- function(
   if (length(tmp2) > 0) tmpData <- tmpData[-tmp2, ]
 
   # Intermediate Step: delete cases for which conditions min.val.n.Vars and  min.val.Tpoints are not met
-  #head(tmpData, 40)
   tmp1 <- apply(tmpData[, allOutputVariablesNames], 1, function(x) sum(!(is.na(x))))
   tmp2 <- which(tmp1 < min.val.n.Vars)
   if(length(tmp2) > 0 ) tmpData <- tmpData[-tmp2, ]
@@ -371,10 +442,33 @@ ctmaShapeRawData <- function(
     tmp3 <- which(tmp2 == 0)
     validTpoints[tmp3, i+1] <- 0
   }
-  #head(validTpoints, 40)
   tmp1 <- apply(validTpoints, 1, function(x) sum(x))
   tmp2 <- which(tmp1 < min.val.Tpoints)
   if(length(tmp2) > 0 ) tmpData <- tmpData[-tmp2, ]
+  #head(tmpData)
+  #apply(tmpData, 2, sd, na.rm=T)
+
+  # CHD 3.11.2023
+  ### Step 2b - standardize within time points if requested
+  if (standardization == "withintimea") {
+    Msg <- "Variables are standardized within time points. This implies that all cases will be deleted that have missing target variables at T0.\n"
+    message(Msg)
+    tmp1 <- grep("_T0", colnames(tmpData)); tmp1
+    tmp2 <- apply(tmpData[, tmp1], 1, sum, na.rm=T)
+    tmp3 <- which(tmp2 == 0); head(tmp3)
+    if (length(tmp3) > 0) tmpData <- tmpData[-tmp3, ]
+    for (c in allOutputVariablesNames) {
+      tmpData[, c] <- scale(tmpData[, c])
+    }
+  }
+  ### Step 2b - standardize within time points if requested
+  if (standardization == "withintimeb") {
+    Msg <- "Variables are standardized within time points. All cases are retained. The user is adviced to use 'sameInitialTimes=TRUE' in ctsem or CoTiMA applications.\n"
+    message(Msg)
+    for (c in allOutputVariablesNames) {
+      tmpData[, c] <- scale(tmpData[, c])
+    }
+  }
 
 
   # Step 6e - Shift data left if 1st time point is missing (otherwise lags will be not computed correctly later)
@@ -399,13 +493,12 @@ ctmaShapeRawData <- function(
     tmpData2[tmp1, allOutputTimeVariablesNames[1:(Tpoints-t)]] <-  tmpData2[tmp1, allOutputTimeVariablesNames[(2):(Tpoints-t+1)]]
     tmpData2[tmp1, allOutputTimeVariablesNames[Tpoints+1-t]] <- NA
   }
+  #head(tmpData2)
   tmpData <- tmpData2
-  #head(tmpData)
 
   # Step 6 Shift data left if all process variables are missing at a time point (even if time stamp is available)
-  #if (experimental == TRUE) {
   if (Tpoints > 2) {
-  for (tt in 2:(Tpoints-1)) {
+    for (tt in 2:(Tpoints-1)) {
       for (t in tt:(Tpoints-1)) {
         # which substantive T1 variables are all missing
         tmp2 <- which(is.na(tmpData2[, allOutputVariablesNames[((tt-1)*(n.manifest)+1):((tt-1)*(n.manifest)+n.manifest)]]), arr.ind = TRUE)
@@ -426,18 +519,20 @@ ctmaShapeRawData <- function(
         tmp2 <- as.numeric(names(tmp2))
         tmpData2[tmp2, allOutputTimeVariablesNames[(Tpoints)]] <- NA
       }
-
-      # delete time stamps and TDpreds if process variables are missing
-      for (t in tt:(Tpoints-0)) {
-        tmp2 <- which(is.na(tmpData2[, allOutputVariablesNames[((t-1)*n.manifest+1):(t*n.manifest)]]), arr.ind = TRUE)
-        tmp2 <- which(table(tmp2[, 1]) == n.manifest)
-        tmp2 <- as.numeric(names(tmp2))
-        tmpData2[tmp2, allOutputTimeVariablesNames[(t)]] <- NA
-      }
     }
-    tmpData <- tmpData2
+
+    # delete time stamps and TDpreds if process variables are missing
+    for (t in tt:(Tpoints-0)) {
+      tmp2 <- which(is.na(tmpData2[, allOutputVariablesNames[((t-1)*n.manifest+1):(t*n.manifest)]]), arr.ind = TRUE)
+      tmp2 <- which(table(tmp2[, 1]) == n.manifest)
+      tmp2 <- as.numeric(names(tmp2))
+      tmpData2[tmp2, allOutputTimeVariablesNames[(t)]] <- NA
+    }
   }
+  tmpData <- tmpData2
   #head(tmpData)
+  #apply(tmpData, 2, sd, na.rm=T)
+
 
   ### Step 6f - Determine possible lags that
   # - are longer than maxTolDelta
@@ -476,31 +571,62 @@ ctmaShapeRawData <- function(
       }
     }
   }
+  #apply(tmpData, 2, sd, na.rm=T)
 
-  #tmpData3 <- tmpData
-  #tmpData <- tmpData3
+  # further standardization
+  if (standardization == "withincolumn") {
+    targetCols <- c()
+    for (i in outputVariablesNames) targetCols <- c(targetCols, grep(i, colnames(tmpData)))
+    for (i in targetCols) tmpData[, i] <- scale(tmpData[, i])
+  }
+  #
+  if (standardization == "withinperson") {
+    for (i in outputVariablesNames) {
+      #i <- outputVariablesNames[1]; i
+      targetCols <- grep(i, colnames(tmpData)); targetCols
+      #tmpData[1:2, targetCols]
+      tmpData[ , targetCols] <- t(apply(tmpData[ , targetCols], 1, scale))
+    }
+  }
+  #
+  if (standardization == "overall") {
+    for (i in outputVariablesNames) {
+      targetCols <- grep(i, colnames(tmpData)); targetCols
+      overallM <- mean(unlist(tmpData[, targetCols]), na.rm=T); overallM
+      overallSD <- sd(unlist(tmpData[, targetCols]), na.rm=T); overallSD
+      tmpData[ , targetCols] <- (tmpData[ , targetCols] - overallM)/overallSD
+    }
+  }
   #head(tmpData)
+  #round(apply(tmpData, 2, mean, na.rm=T), 2)
+  #round(apply(tmpData, 2, sd, na.rm=T), 2)
+
   if (! ((outputDataFrameFormat == "wide") & (outputTimeFormat == "time")) ) { # do nothing if it is wide and time (except possibly changing time name at the end)
 
     #ctIntervalise requires datawide
     #ctWideToLong requires datawide
     #ctDeintervalise requires datalong
+    #tmpData2 <- tmpData
+    #tmpData <- tmpData2
+    #head(tmpData)
 
     tmpData <- ctIntervalise(datawide=tmpData,
-                              Tpoints=Tpoints,
-                              n.manifest=n.manifest,
-                              n.TDpred = n.TDpred,
-                              n.TIpred = n.TIpred,
-                              manifestNames = newOutputVariablesNames,
-                              TDpredNames = generalTDpredNames,
-                              TIpredNames = outputTIpredNames)
+                             Tpoints=Tpoints,
+                             n.manifest=n.manifest,
+                             n.TDpred = n.TDpred,
+                             n.TIpred = n.TIpred,
+                             manifestNames = newOutputVariablesNames,
+                             TDpredNames = generalTDpredNames,
+                             TIpredNames = outputTIpredNames)
+
     if (outputDataFrameFormat == "long")  {
       # without intervalising it does not work correctly
       tmpData <- ctsem::ctWideToLong(tmpData, Tpoints = Tpoints, n.manifest = n.manifest,
-                                           n.TDpred = n.TDpred, n.TIpred = n.TIpred,
-                                           manifestNames =  newOutputVariablesNames,
-                                           TDpredNames = generalTDpredNames, TIpredNames = outputTIpredNames)
+                                     n.TDpred = n.TDpred, n.TIpred = n.TIpred,
+                                     manifestNames =  newOutputVariablesNames,
+                                     TDpredNames = generalTDpredNames, TIpredNames = outputTIpredNames)
       tmpData <- data.frame(tmpData)
+
       # delete cases where time is missing
       tmp1 <- which(tmpData$dT == minInterval)
       if (length(tmp1) > 0) tmpData <- tmpData[-tmp1, ]
@@ -508,8 +634,11 @@ ctmaShapeRawData <- function(
       tmp1 <- apply(tmpData[, outputVariablesNames], 1, function(x) sum(!(is.na(x))))
       tmp2 <- which(tmp1 == 0)
       if(length(tmp2) > 0 ) tmpData <- tmpData[-tmp2, ]
+      #head(tmpData, 50)
+      #tmpData3 <- tmpData
     }
-    #head(tmpData, 30)
+    #head(tmpData)
+    #apply(tmpData, 2, sd, na.rm=T)
 
     #outputTimeFormat
     if (outputTimeFormat == "time") {
@@ -527,11 +656,12 @@ ctmaShapeRawData <- function(
         tmpData[tmpData$id == i,] <- currentData
       }
     }
-    #head(tmpData, 30)
-
-
-    } # end   if (!(outputDataFrameFormat == "wide") & (outputTimeFormat == "time"))
+    head(tmpData, 50)
+    #apply(tmpData, 2, mean, na.rm=T)
+  } # end   if (!(outputDataFrameFormat == "wide") & (outputTimeFormat == "time"))
   #head(tmpData)
+  #apply(tmpData, 2, sd, na.rm=T)
+
 
   # correction of time names
   if (outputTimeVariablesNames != "time") {
@@ -542,6 +672,6 @@ ctmaShapeRawData <- function(
     tmp1 <- grep("dT", colnames(tmpData)); tmp1
     colnames(tmpData) <- gsub("dT", outputTimeVariablesNames, colnames(tmpData))
   }
-
+  #head(tmpData, 50)
   return(tmpData)
 }
